#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Testing for Tardis
"""

import unittest

from copy import deepcopy
from datetime import date
from datetime import datetime
from datetime import timedelta
from datetime import tzinfo

import mock
import pytz

from dateutil.tz import tzlocal
from dateutil.tz import tzoffset

import tardis


class GenericUTC(tzinfo):
    """GenericUTC"""
    ZERO = timedelta(0)

    def utcoffset(self, dt):
        return self.ZERO

    def tzname(self, dt):
        return "GenericUTC"

    def dst(self, dt):
        return self.ZERO


UTC = "UTC"
generic_utc = GenericUTC()
est = pytz.timezone("US/Eastern").localize(datetime.utcnow()).tzinfo


class TardisTests(unittest.TestCase):
    def setUp(self):
        self.naive_dt = datetime(2013, 1, 3, 4, 31, 14, 148540)
        self.do = tardis.Tardis(datetime=self.naive_dt, timezone="UTC")

    def test_date_failure(self):
        dt = date(2013, 5, 6)
        self.assertRaises(ValueError, tardis.Tardis, dt)

    def test_initialize_from_datetime_naive(self):
        self.assertRaises(tardis.TardisInvalidTimezone, tardis.Tardis, datetime=self.naive_dt)

    def test_initialize_with_tzinfo_generic(self):
        aware_dt_generic = datetime(2013, 1, 3, 4, 31, 14, 148540, tzinfo=generic_utc)
        do = tardis.Tardis(datetime=aware_dt_generic)
        self.assertTrue(isinstance(do, tardis.Tardis))

    def test_initialize_with_tzinfo_pytz(self):
        aware_dt_pytz = datetime(2013, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        do = tardis.Tardis(datetime=aware_dt_pytz)
        self.assertTrue(isinstance(do, tardis.Tardis))

    def test_initialize_with_invalid_datetime(self):
        self.assertRaises(ValueError, tardis.Tardis, '2015-01-01')

    def test_initialize_with_naive_datetime_with_pytz_timezone(self):
        dt = datetime(2015, 1, 1)
        do = tardis.Tardis(dt, timezone=pytz.utc)

        dt = pytz.utc.localize(dt)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, pytz.utc)

    def test_initialize_with_naive_datetime_with_dateutil_timezone(self):
        dt = datetime(2015, 1, 1)
        tz = tzoffset(None, -22500)
        do = tardis.Tardis(dt, timezone=tz)

        utcoffset = tz.utcoffset(None)
        total_seconds = (
            (utcoffset.microseconds + (utcoffset.seconds + utcoffset.days * 24 * 3600) * 10 ** 6) / 10 ** 6)
        tz = pytz.FixedOffset(total_seconds / 60)
        dt = tz.localize(dt)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    def test_initialize_with_naive_datetime_with_string_timezone(self):
        dt = datetime(2015, 1, 1)
        tz_str = 'US/Pacific'
        tz = pytz.timezone(tz_str)
        do = tardis.Tardis(dt, timezone=tz_str)
        tz = tz.localize(datetime(2015, 1, 1)).tzinfo

        dt = tz.localize(dt)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    def test_initialize_with_naive_datetime_without_timezone(self):
        self.assertRaises(tardis.TardisInvalidTimezone, tardis.Tardis, datetime(2015, 1, 1))

    def test_initialize_with_datetime_with_timezone(self):
        dt = pytz.utc.localize(datetime(2015, 1, 1))
        do = tardis.Tardis(dt, timezone=pytz.timezone('US/Pacific'))

        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, pytz.utc)

    def test_initialize_with_datetime_without_timezone(self):
        dt = pytz.utc.localize(datetime(2015, 1, 1))
        do = tardis.Tardis(dt)

        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, pytz.utc)

    @mock.patch('tardis.dates.datetime_timezone')
    def test_initialize_without_datetime_with_pytz_timezone(self, mock_datetime_timezone):
        tz = pytz.timezone('US/Pacific')
        dt = tz.localize(datetime(2015, 1, 1))
        tz = dt.tzinfo
        mock_datetime_timezone.return_value = dt

        do = tardis.Tardis(timezone=tz)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    @mock.patch('tardis.dates.datetime_timezone')
    def test_initialize_without_datetime_with_dateutil_timezone(self, mock_datetime_timezone):
        tz = tzoffset(None, -22500)
        dt = datetime(2015, 1, 1, tzinfo=tz)

        utcoffset = tz.utcoffset(None)
        total_seconds = (
            (utcoffset.microseconds + (utcoffset.seconds + utcoffset.days * 24 * 3600) * 10 ** 6) / 10 ** 6)

        tz = pytz.FixedOffset(total_seconds / 60)
        dt = tz.normalize(dt)
        mock_datetime_timezone.return_value = dt

        do = tardis.Tardis(timezone=tz)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    @mock.patch('tardis.dates.datetime_timezone')
    def test_initialize_without_datetime_with_string_timezone(self, mock_datetime_timezone):
        tz_str = 'US/Pacific'
        tz = pytz.timezone(tz_str)
        dt = tz.localize(datetime(2015, 1, 1))
        tz = dt.tzinfo
        mock_datetime_timezone.return_value = dt

        do = tardis.Tardis(timezone=tz_str)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    @mock.patch('tardis.dates.datetime_timezone')
    def test_initialize_without_datetime_without_timezone(self, mock_datetime_timezone):
        dt = pytz.utc.localize(datetime(2015, 1, 1))
        mock_datetime_timezone.return_value = dt

        do = tardis.Tardis()
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, pytz.utc)

    def test_truncation_hour(self):
        self.do.truncate('hour')
        self.assertEqual(self.do.naive, datetime(2013, 1, 3, 4, 0))

    def test_midnight(self):
        dt = self.do.midnight
        self.assertEqual(dt, datetime(2013, 1, 3, 0, 0, 0, tzinfo=pytz.utc))

    def test_start_of_day(self):
        dt = self.do.start_of_day
        self.assertEqual(dt, datetime(2013, 1, 3, 0, 0, 0, 0, tzinfo=pytz.utc))

    def test_end_of_day(self):
        dt = self.do.end_of_day
        self.assertEqual(dt, datetime(2013, 1, 3, 23, 59, 59, 999999, tzinfo=pytz.utc))

    def test_truncation_second(self):
        self.do.truncate('second')
        self.assertEqual(self.do.naive, datetime(2013, 1, 3, 4, 31, 14, 0))

    def test_truncation_minute(self):
        self.do.truncate('minute')
        self.assertEqual(self.do.naive, datetime(2013, 1, 3, 4, 31, 0, 0))

    def test_truncation_day(self):
        self.do.truncate('day')
        self.assertEqual(self.do.naive, datetime(2013, 1, 3, 0, 0, 0, 0))

    def test_truncation_month(self):
        self.do.truncate('month')
        self.assertEqual(self.do.naive, datetime(2013, 1, 1, 0, 0, 0, 0))

    def test_truncation_year(self):
        self.do.truncate('year')
        self.assertEqual(self.do.naive, datetime(2013, 1, 1, 0, 0, 0, 0))

    def test_date(self):
        self.assertEqual(self.do.date, date(2013, 1, 3))

    def test_datetime(self):
        self.assertEqual(self.do.naive, datetime(2013, 1, 3, 4, 31, 14, 148540))

    def test_naive(self):
        dt1 = tardis.Tardis()
        dt_naive = dt1.naive
        self.assertEqual(dt_naive.tzinfo, None)

    def test_naive_timezone(self):
        dt1 = tardis.Tardis(timezone="US/Eastern").truncate('minute').naive
        dt2 = tardis.Tardis().truncate('minute').naive
        self.assertEqual(dt2, dt1)
        self.assertEqual(dt1.tzinfo, None)

    def test_localize(self):
        dt = datetime.today()
        dt = tardis.localize(dt, "UTC")
        self.assertEqual(dt.tzinfo, pytz.utc)

    def test_failure_truncation(self):
        self.assertRaises(ValueError, self.do.truncate, "century")

    def test_normalize(self):
        dt1 = tardis.Tardis()
        dt2 = tardis.Tardis(timezone="US/Eastern")
        dt1.truncate('minute')
        dt2.truncate('minute')
        dt_normalized = tardis.normalize(dt1.datetime, "US/Eastern")
        self.assertEqual(dt2.datetime, dt_normalized)

    def test_normalize_failure(self):
        naive_datetime = datetime.today()
        self.assertRaises(ValueError, tardis.normalize, naive_datetime, "US/Eastern")

    def test_localize_failure(self):
        dt1 = tardis.localize(datetime.utcnow(), "UTC")
        self.assertRaises(ValueError, tardis.localize, dt1, "UTC")

    def test_timezone(self):
        do_timezone = tardis.Tardis().timezone
        self.assertEqual(pytz.utc, do_timezone)

    def test_datetime_timezone_default(self):
        do = tardis.Tardis()
        do.truncate('minute')
        dt1 = tardis.datetime_timezone(UTC)
        self.assertEqual(dt1.replace(second=0, microsecond=0), do.datetime)

    def test_datetime_timezone(self):
        do = tardis.Tardis(timezone="US/Eastern")
        do.truncate("minute")
        dt1 = tardis.datetime_timezone(tz="US/Eastern")
        self.assertEqual(dt1.replace(second=0, microsecond=0), do.datetime)

    def test_parse(self):
        do = tardis.parse('Thu Sep 25 10:36:28 BRST 2003')
        dt1 = pytz.utc.localize(datetime(2003, 9, 25, 10, 36, 28))
        self.assertEqual(do.datetime, dt1)

    def test_parse_default(self):
        do = tardis.parse('2016-07-01T11:00:00+02:00', dayfirst=False)
        dt1 = tardis.Tardis(datetime=datetime(2016, 7, 1, 11, 0), timezone=pytz.FixedOffset(120))
        self.assertEqual(do.datetime, dt1.datetime)

    def test_parse_with_invalid_datetime_string(self):
        self.assertRaises(ValueError, tardis.parse, 'asd')

    def test_parse_with_utc_year_fill(self):
        do = tardis.parse('Thu Sep 25 10:36:28')
        dt1 = pytz.utc.localize(datetime(date.today().year, 9, 25, 10, 36, 28))
        self.assertEqual(do.datetime, dt1)

    def test_parse_with_timezone_year_fill(self):
        do = tardis.parse('Thu Sep 25 10:36:28')
        dt1 = pytz.utc.localize(datetime(date.today().year, 9, 25, 10, 36, 28))
        self.assertEqual(do.datetime, dt1)
        self.assertEqual(do.timezone.tzname(None), "UTC")

    def test_parse_with_fixed_offset_timezone(self):
        tz = pytz.FixedOffset(-480)
        dt = tz.localize(datetime(2015, 1, 1))
        dt_str = dt.strftime('%Y-%m-%d %H:%M:%S %z')

        do = tardis.parse(dt_str)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    @mock.patch('tardis.interface.get_localzone')
    def test_parse_with_tzlocal_timezone(self, mock_get_local_zone):
        tz = pytz.timezone('US/Eastern')
        mock_get_local_zone.return_value = tz
        dt = datetime(2015, 1, 1, tzinfo=tzlocal())
        dt_str = dt.strftime('%Y-%m-%d %H:%M:%S %Z')
        dt = dt.replace(tzinfo=None)
        dt = tz.localize(dt)
        tz = dt.tzinfo

        do = tardis.parse(dt_str)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    @mock.patch('tardis.interface.get_localzone')
    def test_parse_with_tzutc_timezone(self, mock_get_localzone):
        mock_get_localzone.return_value = pytz.utc
        dt = pytz.utc.localize(datetime(2015, 1, 1))
        dt_str = dt.strftime('%Y-%m-%d %H:%M:%S %Z')

        do = tardis.parse(dt_str)
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, pytz.utc)

    def test_parse_with_timezone_parameter(self):
        tz = pytz.timezone('US/Pacific')
        dt = tz.localize(datetime(2015, 1, 1))
        tz = dt.tzinfo
        dt_str = dt.strftime('%Y-%m-%d %H:%M:%S')

        do = tardis.parse(dt_str, timezone='US/Pacific')
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    def test_parse_with_overriding_timezone_parameter(self):
        tz = pytz.timezone('US/Pacific')
        dt = tz.localize(datetime(2015, 1, 1))
        tz = dt.tzinfo
        dt_str = dt.strftime('%Y-%m-%d %H:%M:%S -0500')

        do = tardis.parse(dt_str, timezone='US/Pacific')
        self.assertEqual(do.datetime, dt)
        self.assertEqual(do.timezone, tz)

    def test_move_namedday(self):
        dt_next = datetime(2013, 1, 4, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2013, 1, 11, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 12, 28, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2012, 12, 21, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_friday()
        d_obj_next_2 = self.do.next_friday(2)
        d_obj_last = self.do.last_friday()
        d_obj_last_2 = self.do.last_friday(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_namedday_function(self):
        dt_next = datetime(2013, 1, 4, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 12, 28, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_namedday(self.do.datetime, 'next', 'friday')
        d_obj_last = tardis.move_datetime_namedday(self.do.datetime, 'last', 'friday')

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_week(self):
        dt_next = datetime(2013, 1, 10, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2013, 1, 17, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 12, 27, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2012, 12, 20, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_week()
        d_obj_next_2 = self.do.next_week(2)
        d_obj_last = self.do.last_week()
        d_obj_last_2 = self.do.last_week(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_week_function(self):
        dt_next = datetime(2013, 1, 10, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 12, 27, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_week(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_week(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_month(self):
        dt_next = datetime(2013, 2, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2013, 3, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 12, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2012, 11, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_month()
        d_obj_next_2 = self.do.next_month(2)
        d_obj_last = self.do.last_month()
        d_obj_last_2 = self.do.last_month(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_month_function(self):
        dt_next = datetime(2013, 2, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 12, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_month(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_month(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_day_function(self):
        dt_next = datetime(2013, 1, 4, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 2, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_day(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_day(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_year(self):
        dt_next = datetime(2014, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2015, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2011, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_year()
        d_obj_next_2 = self.do.next_year(2)
        d_obj_last = self.do.last_year()
        d_obj_last_2 = self.do.last_year(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_year_function(self):
        dt_next = datetime(2014, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2012, 1, 3, 4, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_year(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_year(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_hour(self):
        dt_next = datetime(2013, 1, 3, 5, 31, 14, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2013, 1, 3, 6, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 3, 3, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2013, 1, 3, 2, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_hour()
        d_obj_next_2 = self.do.next_hour(2)
        d_obj_last = self.do.last_hour()
        d_obj_last_2 = self.do.last_hour(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_hour_function(self):
        dt_next = datetime(2013, 1, 3, 5, 31, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 3, 3, 31, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_hour(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_hour(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_minute(self):
        dt_next = datetime(2013, 1, 3, 4, 32, 14, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2013, 1, 3, 4, 33, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 3, 4, 30, 14, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2013, 1, 3, 4, 29, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_minute()
        d_obj_next_2 = self.do.next_minute(2)
        d_obj_last = self.do.last_minute()
        d_obj_last_2 = self.do.last_minute(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_minute_function(self):
        dt_next = datetime(2013, 1, 3, 4, 32, 14, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 3, 4, 30, 14, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_minute(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_minute(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_move_shift_minute(self):
        dt_next = datetime(2013, 1, 3, 4, 31, 15, 148540, tzinfo=pytz.utc)
        dt_next_2 = datetime(2013, 1, 3, 4, 31, 16, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 3, 4, 31, 13, 148540, tzinfo=pytz.utc)
        dt_last_2 = datetime(2013, 1, 3, 4, 31, 12, 148540, tzinfo=pytz.utc)

        d_obj_next = self.do.next_second()
        d_obj_next_2 = self.do.next_second(2)
        d_obj_last = self.do.last_second()
        d_obj_last_2 = self.do.last_second(2)

        self.assertEqual(dt_next, d_obj_next.datetime)
        self.assertEqual(dt_last, d_obj_last.datetime)
        self.assertEqual(dt_next_2, d_obj_next_2.datetime)
        self.assertEqual(dt_last_2, d_obj_last_2.datetime)

    def test_move_second_function(self):
        dt_next = datetime(2013, 1, 3, 4, 31, 15, 148540, tzinfo=pytz.utc)
        dt_last = datetime(2013, 1, 3, 4, 31, 13, 148540, tzinfo=pytz.utc)

        d_obj_next = tardis.move_datetime_second(self.do.datetime, 'next', 1)
        d_obj_last = tardis.move_datetime_second(self.do.datetime, 'last', 1)

        self.assertEqual(dt_next, d_obj_next)
        self.assertEqual(dt_last, d_obj_last)

    def test_range_count(self):
        """
        tests the range method with count used
        """
        count = list(tardis.stops(tardis.DAILY, count=5))
        self.assertEqual(len(count), 5)

    def test_range_with_start(self):
        dates1 = []
        for do in tardis.stops(tardis.DAILY, count=5, start=datetime.utcnow()):
            do.truncate('minute')
            dates1.append(do)
        do = tardis.Tardis().truncate('minute')
        dates2 = []
        for x in range(5):
            dates2.append(do.next_day(x))
        self.assertEqual(dates1, dates2)

    def test_range_with_start_and_stop(self):
        dates1 = []
        tomorrow = datetime.utcnow() + timedelta(days=1)
        for do in tardis.stops(tardis.DAILY, start=datetime.utcnow(), stop=tomorrow):
            do.truncate('minute')
            dates1.append(do)
        do = tardis.Tardis().truncate('minute')
        dates2 = []
        for x in range(2):
            dates2.append(do.next_day(x))
        self.assertEqual(dates1, dates2)

    def test_range_with_interval(self):
        dates1 = []
        for do in tardis.stops(tardis.DAILY, interval=2, count=3, start=datetime.utcnow()):
            do.truncate('minute')
            dates1.append(do)
        do = tardis.Tardis().truncate('minute')
        dates2 = []
        for x in range(6):
            if (x % 2) == 0:
                dates2.append(do.next_day(x))
        self.assertEqual(dates1, dates2)

    def test_tardis_with_datetime(self):
        dt = datetime.utcnow()
        d = tardis.Tardis(datetime=dt, timezone=UTC)
        dt = pytz.utc.localize(dt)
        self.assertEqual(dt, d._dt)
        self.assertEqual(pytz.utc, d.timezone)

    def test_tardis_with_timezone(self):
        dt = datetime.utcnow()
        d = tardis.Tardis(datetime=dt, timezone=UTC)
        d = d.shift("US/Eastern")
        dt = pytz.utc.localize(dt).astimezone(est)
        dt = est.normalize(dt)
        self.assertEqual(dt, d.datetime)
        self.assertEqual(est, d.timezone)

    def test_tardis_with_only_timezone(self):
        dt = datetime.utcnow()
        dt = pytz.utc.localize(dt)
        dt = est.normalize(dt)
        dt = dt.replace(second=0, microsecond=0)
        d = tardis.Tardis(timezone="US/Eastern")
        d.truncate('minute')
        self.assertEqual(est, d.timezone)
        self.assertEqual(dt, d._dt)

    def test_shift_failure(self):
        self.assertRaises(tardis.TardisInvalidTimezone, self.do.shift, "US/Westerrn")

    def test_datetime_localization(self):
        dt1 = self.do.datetime
        dt2 = tardis.Tardis(dt1).datetime
        self.assertEqual(dt1, dt2)

    def test_localize_datetime(self):
        dt = datetime.utcnow()
        tz = pytz.timezone("US/Pacific")
        dt = tz.localize(dt)
        d = tardis.Tardis(dt)
        d2 = d.shift('US/Pacific')

        self.assertEqual(d.timezone.tzname(None), "US/Pacific")
        self.assertEqual(d.datetime, dt)
        self.assertEqual(d.datetime, d2.datetime)

    def test_lt(self):
        dt1 = self.do
        dt2 = tardis.Tardis()
        self.assertTrue(dt1 < dt2)

    def test_gt(self):
        dt1 = self.do
        dt2 = tardis.Tardis()
        self.assertTrue(dt2 > dt1)

    def test_ge(self):
        dt = datetime.utcnow()
        dt1 = tardis.Tardis(dt, timezone="UTC")
        dt2 = tardis.Tardis(dt, timezone="UTC")
        dt3 = self.do
        self.assertTrue(dt2 >= dt1)
        self.assertTrue(dt1 >= dt3)

    def test_le(self):
        dt = datetime.utcnow()
        dt1 = tardis.Tardis(dt, timezone="UTC")
        dt2 = tardis.Tardis(dt, timezone="UTC")
        dt3 = self.do
        self.assertTrue(dt2 <= dt1)
        self.assertTrue(dt3 <= dt2)

    def test_epoch(self):
        unix_time = self.do.epoch
        self.assertEqual(unix_time, 1357187474.148540)

    def test_epoch_creation(self):
        do = tardis.epoch(1357187474.148540)
        self.assertEqual(self.do, do)

    def test_not_equal(self):
        d = tardis.Tardis()
        self.assertNotEqual(d, None)

    def test_equal(self):
        d1 = tardis.Tardis()
        d2 = deepcopy(d1)
        self.assertEqual(d1, d2)
        self.assertFalse(d1 != d2, 'Overloaded __ne__ is not correct')

        d1 = tardis.Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
        d2 = tardis.Tardis(datetime(2015, 1, 1, 8), timezone='UTC')
        self.assertEqual(d1, d2)

    def test_repr_string_timezone(self):
        import datetime
        from tardis import Tardis

        d1 = Tardis(datetime.datetime(2015, 1, 1), timezone='US/Pacific')
        d2 = eval(repr(d1))

        self.assertEqual(d1, d2)

        d3 = Tardis(d1.datetime, timezone='UTC')
        d4 = eval(repr(d3))

        self.assertEqual(d1, d4)

    def test_repr_pytz_timezone(self):
        import datetime
        from tardis import Tardis
        d1 = Tardis(datetime.datetime(2015, 1, 1), timezone='US/Pacific')
        d2 = eval(repr(d1))

        self.assertEqual(d1, d2)
        self.assertEqual(d1.datetime, d2.datetime)
        self.assertEqual(d1.timezone, d2.timezone)

    def test_timezone_tardis_to_datetime_to_tardis_utc(self):
        d1 = tardis.Tardis()
        d2 = tardis.Tardis(d1.datetime)

        # these tardis should be the same
        self.assertEqual(d1.next_day(1), d2.next_day(1))
        self.assertEqual(d2.last_week(), d2.last_week())
        self.assertEqual(d1.timezone, d2.timezone)
        self.assertEqual(d1, d2)

    def test_timezone_tardis_to_datetime_to_tardis_non_utc(self):
        """Test if when you create Tardis object from Tardis' datetime
        it still behaves the same
        """
        d1 = tardis.Tardis(timezone='America/Chicago')
        d2 = tardis.Tardis(d1.datetime)

        # these tardis should be the same
        self.assertEqual(d1.next_day(1), d2.next_day(1))
        self.assertEqual(d2.last_week(), d2.last_week())
        self.assertEqual(d1.timezone, d2.timezone)
        self.assertEqual(d1, d2)

    def test_stops_bymonth(self):
        """Test if create stops, checks bymonth, bymonthday, count
        and start parameters work properly
        """
        days = list(tardis.interface.stops(
            tardis.MONTHLY,
            bymonth=(1, 4, 7, 10),
            bymonthday=15,
            count=4,
            start=datetime(datetime.now().year, 1, 1))
        )
        year = datetime.now().year
        day = 15
        dt1 = datetime(year, 1, day)
        dt4 = datetime(year, 4, day)
        dt7 = datetime(year, 7, day)
        dt10 = datetime(year, 10, day)

        self.assertTrue(len(days) == 4)
        dl1 = tardis.Tardis(datetime=dt1, timezone='UTC')
        self.assertEqual(days[0], dl1)

        dl4 = tardis.Tardis(datetime=dt4, timezone='UTC')
        self.assertEqual(days[1], dl4)

        dl7 = tardis.Tardis(datetime=dt7, timezone='UTC')
        self.assertEqual(days[2], dl7)

        dl10 = tardis.Tardis(datetime=dt10, timezone='UTC')
        self.assertEqual(days[3], dl10)

    def test_timedelta_arithmetic(self):
        hour = timedelta(hours=1)
        d = tardis.parse("2014/06/02 10:00:00 -0700")
        hour_before = tardis.parse("2014/06/02 09:00:00 -0700")
        hour_after = tardis.parse("2014/06/02 11:00:00 -0700")
        self.assertEqual(d + hour, hour_after)
        self.assertEqual(d - hour, hour_before)
        self.assertEqual(hour_after - d, hour)

    @mock.patch('tardis.Tardis.now')
    def test_humanize_past(self, mock_now):
        mock_now.return_value = tardis.Tardis(datetime(2015, 1, 2), timezone='US/Pacific')

        do = tardis.Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
        self.assertEqual(do.humanize(), 'a day ago')

    @mock.patch('tardis.Tardis.now')
    def test_humanize_future(self, mock_now):
        mock_now.return_value = tardis.Tardis(datetime(2014, 12, 31), timezone='US/Pacific')

        do = tardis.Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
        self.assertEqual(do.humanize(), 'a day from now')

    def test_replace(self):
        do = tardis.Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
        dt = do.datetime

        # Check if parts are okay
        self.assertEqual(do.replace(hour=8).datetime.hour, 8)
        self.assertEqual(do.replace(minute=23).datetime.minute, 23)
        self.assertEqual(do.replace(second=45).datetime.second, 45)

        # Asserts on datetimes
        self.assertEqual(do.replace(hour=8).datetime, dt.replace(hour=8))
        self.assertEqual(do.replace(minute=38).datetime, dt.replace(minute=38))
        self.assertEqual(do.replace(second=48).datetime, dt.replace(second=48))

        # Test that the timezone does not change
        self.assertEqual(do.replace(second=45).timezone, do.timezone)

        # Test the datetime
        eight_hours_late = dt + timedelta(hours=8)
        self.assertEqual(do.replace(hour=8).datetime, eight_hours_late)


if __name__ == '__main__':
    unittest.main()



from dateutil.rrule import (
                            YEARLY, MONTHLY, WEEKLY, DAILY, HOURLY,
                            MINUTELY, SECONDLY, MO, TU, WE, TH, FR,
                            SA, SU
)

from tardis.dates import (
                   move_datetime_second, move_datetime_minute,
                   move_datetime_hour, move_datetime_day,
                   move_datetime_week, move_datetime_month,
                   move_datetime_year, move_datetime_namedday,
                   Tardis, datetime_timezone, localize, normalize
)
from tardis.exceptions import TardisInvalidTimezone, TardisInvalidDatetime
from tardis.interface import (
                        parse, stops, epoch, flux, utcnow, now,
                        range_hourly, range_daily, range_monthly,
                        range_yearly
)



import sys

from datetime import datetime
from datetime import timedelta
from datetime import tzinfo
from functools import partial
from functools import update_wrapper

import humanize
import pytz

from babel.dates import format_datetime
from dateutil.tz import tzoffset
from dateutil.relativedelta import relativedelta
from tzlocal import get_localzone

from .exceptions import TardisInvalidTimezone


def get_total_second(td):
    """
    This method takes a timedelta and return the number of seconds it
    represents with the resolution of 10**6
    """
    return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 1e6) / 1e6


def is_datetime_naive(dt):
    """
    This method returns true if the datetime is naive else returns false
    """
    if dt.tzinfo is None:
        return True
    else:
        return False


def is_datetime_instance(dt):
    if dt is None:
        return
    if not isinstance(dt, datetime):
        raise ValueError('Please provide a datetime instance to Tardis')


def _move_datetime(dt, direction, delta):
    """
    Move datetime given delta by given direction
    """
    if direction == 'next':
        dt = dt + delta
    elif direction == 'last':
        dt = dt - delta
    else:
        pass
        # raise some tardis error here
    return dt


def move_datetime_day(dt, direction, num_shifts):
    delta = relativedelta(days=+num_shifts)
    return _move_datetime(dt, direction, delta)


def move_datetime_namedday(dt, direction, unit):
    TOTAL_DAYS = 7
    days = {
        'monday': 1,
        'tuesday': 2,
        'wednesday': 3,
        'thursday': 4,
        'friday': 5,
        'saturday': 6,
        'sunday': 7,
    }

    current_day = days[dt.strftime('%A').lower()]
    target_day = days[unit.lower()]

    if direction == 'next':
        if current_day < target_day:
            delta_days = target_day - current_day
        else:
            delta_days = (target_day - current_day) + TOTAL_DAYS
    elif direction == 'last':

        if current_day <= target_day:
            delta_days = (current_day - target_day) + TOTAL_DAYS
        else:
            delta_days = current_day - target_day

    delta = relativedelta(days=+delta_days)
    return _move_datetime(dt, direction, delta)


def move_datetime_month(dt, direction, num_shifts):
    """
    Move datetime 1 month in the chosen direction.
    unit is a no-op, to keep the API the same as the day case
    """
    delta = relativedelta(months=+num_shifts)
    return _move_datetime(dt, direction, delta)


def move_datetime_week(dt, direction, num_shifts):
    """
    Move datetime 1 week in the chosen direction.
    unit is a no-op, to keep the API the same as the day case
    """
    delta = relativedelta(weeks=+num_shifts)
    return _move_datetime(dt, direction, delta)


def move_datetime_year(dt, direction, num_shifts):
    """
    Move datetime 1 year in the chosen direction.
    unit is a no-op, to keep the API the same as the day case
    """
    delta = relativedelta(years=+num_shifts)
    return _move_datetime(dt, direction, delta)


def move_datetime_hour(dt, direction, num_shifts):
    delta = relativedelta(hours=+num_shifts)
    return _move_datetime(dt, direction, delta)


def move_datetime_minute(dt, direction, num_shifts):
    delta = relativedelta(minutes=+num_shifts)
    return _move_datetime(dt, direction, delta)


def move_datetime_second(dt, direction, num_shifts):
    delta = relativedelta(seconds=+num_shifts)
    return _move_datetime(dt, direction, delta)


def datetime_timezone(tz):
    """
    This method given a timezone returns a localized datetime object.
    """
    utc_datetime_naive = datetime.utcnow()
    # return a localized datetime to UTC
    utc_localized_datetime = localize(utc_datetime_naive, 'UTC')
    # normalize the datetime to given timezone
    normalized_datetime = normalize(utc_localized_datetime, tz)
    return normalized_datetime


def localize(dt, tz):
    """
    Given a naive datetime object this method will return a localized
    datetime object
    """
    if not isinstance(tz, tzinfo):
        tz = pytz.timezone(tz)

    return tz.localize(dt)


def normalize(dt, tz):
    """
    Given a object with a timezone return a datetime object
    normalized to the proper timezone.

    This means take the give localized datetime and returns the
    datetime normalized to match the specificed timezone.
    """
    if not isinstance(tz, tzinfo):
        tz = pytz.timezone(tz)
    dt = tz.normalize(dt)
    return dt


class Tardis(object):
    """
    The class `Tardis <Tardis>` object. This method accepts naive
    datetime objects, with a string timezone.
    """
    _VALID_SHIFT_DIRECTIONS = ('last', 'next')
    _VALID_SHIFT_UNITS = ('second', 'minute', 'hour', 'day', 'week',
                          'month', 'year', 'monday', 'tuesday', 'wednesday',
                          'thursday', 'friday', 'saturday', 'sunday')

    def __init__(self, datetime=None, timezone=None):
        # maybe set timezone on the way in here. if here set it if not
        # use UTC
        is_datetime_instance(datetime)

        if datetime:
            if is_datetime_naive(datetime):
                if timezone:
                    if isinstance(timezone, tzoffset):
                        utcoffset = timezone.utcoffset(None)
                        total_seconds = (
                            (utcoffset.microseconds + (
                                utcoffset.seconds + utcoffset.days * 24 * 3600) * 10 ** 6) / 10 ** 6)
                        self._tzinfo = pytz.FixedOffset(total_seconds / 60)
                    elif isinstance(timezone, tzinfo):
                        self._tzinfo = timezone
                    else:
                        self._tzinfo = pytz.timezone(timezone)
                    self._dt = localize(datetime, self._tzinfo)
                    self._tzinfo = self._dt.tzinfo
                else:
                    # TODO(mlew, 2015-08-09):
                    # Should we really throw an error here, or should this
                    # default to UTC?)
                    raise TardisInvalidTimezone('Provide a valid timezone')
            else:
                self._tzinfo = datetime.tzinfo
                self._dt = datetime
        else:
            if timezone:
                if isinstance(timezone, tzoffset):
                    self._tzinfo = pytz.FixedOffset(
                        timezone.utcoffset(None).total_seconds() / 60)
                elif isinstance(timezone, tzinfo):
                    self._tzinfo = timezone
                else:
                    self._tzinfo = pytz.timezone(timezone)

                self._dt = datetime_timezone(self._tzinfo)
                self._tzinfo = self._dt.tzinfo
            else:
                self._tzinfo = pytz.utc
                self._dt = datetime_timezone('UTC')

    def __repr__(self):
        dt = self.datetime.replace(tzinfo=None)
        if isinstance(self.timezone, pytz._FixedOffset):
            tz = self.timezone
        else:
            tz = self.timezone.tzname(None)

        return 'Tardis(datetime=%r, timezone=%r)' % (dt, tz)

    def __eq__(self, other):
        if isinstance(other, Tardis):
            return self.epoch == other.epoch
        return False

    def __lt__(self, other):
        return self.epoch < other.epoch

    def __gt__(self, other):
        return self.epoch > other.epoch

    def __ge__(self, other):
        return self.epoch >= other.epoch

    def __le__(self, other):
        return self.epoch <= other.epoch

    def __ne__(self, other):
        return not self == other

    def __add__(self, other):
        if not isinstance(other, timedelta):
            raise TypeError(
                "Tardis objects can only be added with timedelta objects")
        dt = self._dt + other
        return Tardis(datetime=dt, timezone=self.timezone)

    def __sub__(self, other):
        if isinstance(other, timedelta):
            dt = self._dt - other
            return Tardis(datetime=dt, timezone=self.timezone)
        elif isinstance(other, Tardis):
            return self._dt - other._dt
        else:
            raise TypeError(
                "Tardis objects can only be subtracted with timedelta or other Tardis objects")

    def __getattr__(self, name):
        """
        Implement __getattr__ to call `shift_date` function when function
        called does not exist
        """
        func_parts = name.split('_')
        # is the func we are trying to call the right length?
        if len(func_parts) != 2:
            raise AttributeError

        # is the function we are trying to call valid?
        if (func_parts[0] not in self._VALID_SHIFT_DIRECTIONS or
                func_parts[1] not in self._VALID_SHIFT_UNITS):
            return AttributeError

        # dispatch our function
        func = partial(self._shift_date, func_parts[0], func_parts[1])
        # update our partial with self.shift_date attributes
        update_wrapper(func, self._shift_date)
        return func

    def _shift_date(self, direction, unit, *args):
        """
        Shift datetime in `direction` in _VALID_SHIFT_DIRECTIONS and by some
        unit in _VALID_SHIFTS and shift that amount by some multiple,
        defined by by args[0] if it exists
        """
        this_module = sys.modules[__name__]

        num_shifts = 1
        if len(args) > 0:
            num_shifts = int(args[0])

        if unit in ['monday', 'tuesday', 'wednesday', 'thursday', 'friday',
                    'saturday', 'sunday']:
            shift_func = move_datetime_namedday
            dt = shift_func(self._dt, direction, unit)
            if num_shifts > 1:
                for n in range(num_shifts - 1):
                    dt = shift_func(dt, direction, unit)
        else:
            shift_func = getattr(this_module, 'move_datetime_%s' % unit)
            dt = shift_func(self._dt, direction, num_shifts)

        return Tardis(datetime=dt.replace(tzinfo=None), timezone=self.timezone)

    @property
    def timezone(self):
        """
        Returns a valid tzinfo object associated with
        the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1), timezone='UTC')
            >>> d.timezone
            <UTC>
        """
        return self._tzinfo

    def truncate(self, s):
        """
        Truncate the tardis object to the nearest s
        (second, minute, hour, day, month, year)

        This is a destructive method, modifies the internal datetime
        object associated with the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12, 10), timezone='US/Pacific')
            >>> d.truncate('hour')
            Tardis(datetime=datetime.datetime(2015, 1, 1, 12, 0), timezone='US/Pacific')

        """
        if s == 'second':
            self._dt = self._dt.replace(microsecond=0)
        elif s == 'minute':
            self._dt = self._dt.replace(second=0, microsecond=0)
        elif s == 'hour':
            self._dt = self._dt.replace(minute=0, second=0, microsecond=0)
        elif s == 'day':
            self._dt = self._dt.replace(
                hour=0, minute=0, second=0, microsecond=0)
        elif s == 'month':
            self._dt = self._dt.replace(
                day=1, hour=0, minute=0, second=0, microsecond=0)
        elif s == 'year':
            self._dt = self._dt.replace(
                month=1, day=1, hour=0, minute=0, second=0, microsecond=0)
        else:
            raise ValueError("Invalid truncation level")

        return self

    @property
    def naive(self):
        """
        Returns a naive datetime object associated with the Tardis
        object, this method simply converts the localize datetime to UTC
        and removes the tzinfo that is associated with it modifying the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
            >>> d.naive
            datetime.datetime(2015, 1, 1, 8, 0)

        """
        self.shift('UTC')
        return self._dt.replace(tzinfo=None)

    @classmethod
    def now(cls, timezone=None):
        if timezone:
            return cls(timezone=timezone)
        else:
            return cls(timezone=get_localzone())

    @classmethod
    def utcnow(cls):
        return cls()

    @property
    def midnight(self):
        """
        Returns midnight for datetime associated with
        the Tardis object modifying the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12), timezone='UTC')
            >>> d.midnight
            datetime.datetime(2015, 1, 1, 0, 0, tzinfo=<UTC>)

        """
        return self._dt.replace(hour=0, minute=0, second=0, microsecond=0)

    @property
    def start_of_day(self):
        """
        Returns the start of the day for datetime assoicated
        with the Tardis object, modifying the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12), timezone='UTC')
            >>> d.start_of_day
            datetime.datetime(2015, 1, 1, 0, 0, tzinfo=<UTC>)

        """
        return self.midnight

    @property
    def end_of_day(self):
        """
        Returns the end of the day for the datetime
        assocaited with the Tardis object, modifying the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12), timezone='UTC')
            >>> d.end_of_day
            datetime.datetime(2015, 1, 1, 23, 59, 59, 999999, tzinfo=<UTC>)

        """
        return self._dt.replace(hour=23, minute=59, second=59, microsecond=999999)

    def shift(self, timezone):
        """
        Shifts the timezone from the current timezone to the specified timezone associated with the Tardis object,
        modifying the Tardis object and returning the modified object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
            >>> d.shift('UTC')
            Tardis(datetime=datetime.datetime(2015, 1, 1, 8, 0), timezone='UTC')

        """
        try:
            self._tzinfo = pytz.timezone(timezone)
        except pytz.UnknownTimeZoneError:
            raise TardisInvalidTimezone('Provide a valid timezone')
        self._dt = self._tzinfo.normalize(self._dt.astimezone(self._tzinfo))
        self._tzinfo = self._dt.tzinfo
        return self

    @property
    def epoch(self):
        """
        Returns the total seconds since epoch associated with
        the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1), timezone='US/Pacific')
            >>> d.epoch
            1420099200.0

        """
        epoch_sec = pytz.utc.localize(datetime.utcfromtimestamp(0))
        now_sec = pytz.utc.normalize(self._dt)
        delta_sec = now_sec - epoch_sec
        return get_total_second(delta_sec)

    @property
    def date(self):
        """
        Returns the actual date object associated with
        the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12, 15), timezone='US/Pacific')
            >>> d.date
            datetime.date(2015, 1, 1)

        """
        return self._dt.date()

    @property
    def datetime(self):
        """
        Returns the actual datetime object associated with
        the Tardis object.

        .. testsetup::

            from datetime import datetime
            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12, 15), timezone='UTC')
            >>> d.datetime
            datetime.datetime(2015, 1, 1, 12, 15, tzinfo=<UTC>)
        """
        return self._dt

    def replace(self, **kwargs):
        """
            Returns a new Tardis object after applying replace on the
            existing datetime object.

            .. testsetup::

                from datetime import datetime
                from tardis import Tardis

            .. doctest::

                >>> d = Tardis(datetime(2015, 1, 1, 12, 15), timezone='UTC')
                >>> d.replace(hour=8)
                Tardis(datetime=datetime.datetime(2015, 1, 1, 8, 15), timezone='UTC')
            """

        return Tardis(datetime=self._dt.replace(**kwargs), timezone=self.timezone)

    def humanize(self):
        """
        Humanize relative to now:

        .. testsetup::

            from datetime import timedelta
            from tardis import Tardis

        .. doctest::

            >>> past = Tardis.utcnow() - timedelta(hours=1)
            >>> past.humanize()
            'an hour ago'

        """
        now = self.now(self.timezone)

        return humanize.naturaltime(now - self)

    def format_datetime(self, format='medium', locale='en_US'):
        """
        Return a date string formatted to the given pattern.

        .. testsetup::

            from tardis import Tardis

        .. doctest::

            >>> d = Tardis(datetime(2015, 1, 1, 12, 30), timezone='US/Pacific')
            >>> d.format_datetime(locale='en_US')
            u'Jan 1, 2015, 12:30:00 PM'

            >>> d.format_datetime(format='long', locale='de_DE')
            u'1. Januar 2015 12:30:00 -0800'

        :param format: one of "full", "long", "medium", "short", or a custom datetime pattern
        :param locale: a locale identifier

        """
        return format_datetime(self._dt, format=format, locale=locale)



from datetime import datetime

import pytz

from dateutil.rrule import rrule, DAILY, HOURLY, MONTHLY, YEARLY
from dateutil.parser import parse as capture
from dateutil.tz import tzlocal
from dateutil.tz import tzoffset
from tzlocal import get_localzone

from .exceptions import TardisInvalidDatetime
from .dates import Tardis, is_datetime_naive, datetime_timezone


def parse(datetime_str, timezone=None, dayfirst=True, yearfirst=True):
    """
    Parses a datetime string and returns a `Tardis` object.

    :param datetime_str: The string to be interpreted into a `Tardis` object.
    :param timezone: Pass this parameter and the returned Tardis object will be normalized to this timezone. Any
        offsets passed as part of datetime_str will be ignored.
    :param dayfirst: Whether to interpret the first value in an ambiguous 3-integer date (ex. 01/05/09) as the day
        (True) or month (False). If yearfirst is set to True, this distinguishes between YDM and YMD.
    :param yearfirst: Whether to interpret the first value in an ambiguous 3-integer date (ex. 01/05/09) as the
        year. If True, the first number is taken to be the year, otherwise the last number is taken to be the year.

    .. testsetup::

        from tardis import Tardis
        from tardis import parse

    .. doctest::

        >>> parse('2015-01-01 00:01:02')
        Tardis(datetime=datetime.datetime(2015, 1, 1, 0, 1, 2), timezone='UTC')

    If a fixed offset is provided in the datetime_str, it will be parsed and the returned `Tardis` object will store a
    `pytz.FixedOffest` as it's timezone.

    .. doctest::

        >>> parse('2015-01-01 00:01:02 -0800')
        Tardis(datetime=datetime.datetime(2015, 1, 1, 0, 1, 2), timezone=pytz.FixedOffset(-480))

    If the timezone argument is supplied, the returned Tardis object will be in the timezone supplied. Any offsets in
    the datetime_str will be ignored.

    .. doctest::

        >>> parse('2015-01-01 00:01:02 -0500', timezone='US/Pacific')
        Tardis(datetime=datetime.datetime(2015, 1, 1, 0, 1, 2), timezone='US/Pacific')

    If an unambiguous timezone is detected in the datetime string, a Tardis object with that datetime and
    timezone will be returned.

    .. doctest::

        >>> parse('2015-01-01 00:01:02 PST')
        Tardis(datetime=datetime.datetime(2015, 1, 1, 0, 1, 2), timezone='America/Los_Angeles')

    However if the provided timezone is ambiguous, parse will ignore the timezone and return a `Tardis` object in UTC
    time.

        >>> parse('2015-01-01 00:01:02 EST')
        Tardis(datetime=datetime.datetime(2015, 1, 1, 0, 1, 2), timezone='UTC')

    """
    dt = capture(datetime_str, dayfirst=dayfirst, yearfirst=yearfirst)

    if timezone:
        dt = dt.replace(tzinfo=None)
        do = Tardis(datetime=dt, timezone=timezone)
    elif dt.tzinfo is None:
        # assuming datetime object passed in is UTC
        do = Tardis(datetime=dt, timezone='UTC')
    elif isinstance(dt.tzinfo, tzoffset):
        utcoffset = dt.tzinfo.utcoffset(None)
        total_seconds = (
            (utcoffset.microseconds + (utcoffset.seconds + utcoffset.days * 24 * 3600) * 10**6) / 10**6)

        tz = pytz.FixedOffset(total_seconds / 60)
        dt = dt.replace(tzinfo=None)
        do = Tardis(dt, timezone=tz)
    elif isinstance(dt.tzinfo, tzlocal):
        tz = get_localzone()
        dt = dt.replace(tzinfo=None)
        do = Tardis(dt, timezone=tz)
    else:
        dt = pytz.utc.normalize(dt)
        # making dt naive so we can pass it to Tardis
        dt = dt.replace(tzinfo=None)
        # if parse string has tzinfo we return a normalized UTC
        # tardis object that represents the time.
        do = Tardis(datetime=dt, timezone='UTC')

    return do


def range_daily(start=None, stop=None, timezone='UTC', count=None):
    """
    This an alternative way to generating sets of Tardis objects with
    DAILY stops
    """
    return stops(start=start, stop=stop, freq=DAILY, timezone=timezone, count=count)


def range_hourly(start=None, stop=None, timezone='UTC', count=None):
    """
    This an alternative way to generating sets of Tardis objects with
    HOURLY stops
    """
    return stops(start=start, stop=stop, freq=HOURLY, timezone=timezone, count=count)


def range_monthly(start=None, stop=None, timezone='UTC', count=None):
    """
    This an alternative way to generating sets of Tardis objects with
    MONTHLY stops
    """
    return stops(start=start, stop=stop, freq=MONTHLY, timezone=timezone, count=count)


def range_yearly(start=None, stop=None, timezone='UTC', count=None):
    """
    This an alternative way to generating sets of Tardis objects with
    YEARLY stops
    """
    return stops(start=start, stop=stop, freq=YEARLY, timezone=timezone, count=count)


def stops(freq, interval=1, count=None, wkst=None, bysetpos=None,
          bymonth=None, bymonthday=None, byyearday=None, byeaster=None,
          byweekno=None, byweekday=None, byhour=None, byminute=None,
          bysecond=None, timezone='UTC', start=None, stop=None):
    """
    This will create a list of tardis objects the apply to
    setting possed in.
    """
    # check to see if datetimees passed in are naive if so process them
    # with given timezone.
    if all([(start is None or is_datetime_naive(start)),
            (stop is None or is_datetime_naive(stop))]):
        pass
    else:
        raise TardisInvalidDatetime('Provide a naive datetime object')

    # if no datetimes are passed in create a proper datetime object for
    # start default because default in dateutil is datetime.now() :(
    if start is None:
        start = datetime_timezone(timezone)

    for dt in rrule(freq, interval=interval, count=count, wkst=wkst, bysetpos=bysetpos,
          bymonth=bymonth, bymonthday=bymonthday, byyearday=byyearday, byeaster=byeaster,
          byweekno=byweekno, byweekday=byweekday, byhour=byhour, byminute=byminute,
          bysecond=bysecond, until=stop, dtstart=start):
        # make the tardis object
        # yield it.
        # doing this to make sure tardis receives a naive datetime.
        dt = dt.replace(tzinfo=None)
        d = Tardis(datetime=dt, timezone=timezone)
        yield d


def epoch(s):
    dt = datetime.utcfromtimestamp(s)
    return Tardis(datetime=dt, timezone='UTC')


def flux():
    print("If you put your mind to it, you can accomplish anything.")


def utcnow():
    """
    Return a Tardis object for the current UTC date and time, setting the timezone to UTC.
    """
    return Tardis()


def now():
    """
    Return a Tardis object for the current local date and time, setting the timezone to the local timezone of the
    caller.
    """
    return Tardis(timezone=get_localzone())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from version import __version__

dependencies = [
    'babel>=2.1.1',
    'humanize>=0.5.1',
    'python-dateutil>=2.4.2',
    'pytz>=2015.7',
    'tzlocal>=1.2']

setup(
    name='Tardis_time',
    version='.'.join(str(x) for x in __version__),
    description='library for manipulating datetimes with ease and clarity',
    url='https://github.com/urbvantransit/tardis',
    author='Urbvan Technologies',
    author_email="tec@urbvan.com",
    packages=[
    'tardis',
    ],
    license='MIT license',
    install_requires=dependencies,
    test_suite = 'tests.test_data',
    long_description=open('README.rst').read(),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ),
)
